﻿using EPSON_SOCKET;
using Maticsoft.DBUtility;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ZWGXPICK_SYS
{
    public partial class Point_Data_Form : Form
    {
        private float X;
        private float Y;
        private bool isDirty = false; // 数据是否需要保存.
        private EPSON_SOCKET.EPSON_SOCKET epsonSocket1 = null;
        public Point_Data_Form(EPSON_SOCKET.EPSON_SOCKET epsonSocket)
        {
            InitializeComponent();
            this.epsonSocket1 = epsonSocket;
            this.epsonSocket1.rec_data_refresh += this.epsonSocket_rec_data_refresh;
        }

        // 窗口关闭保存参数.
        private void Point_Data_Form_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 当前是否是脏数据.
            if (this.isDirty)
            {
                // 确认是否保存
                DialogResult dr = MessageBox.Show("您好,当前示教点未保存,是否现在保存?", "保存", MessageBoxButtons.YesNo);
                if (dr == DialogResult.Yes)
                {
                    btnSavePoint_Click(sender, e);
                    System.Threading.Thread.Sleep(500);
                }
            }

            // 去掉事件响应并保存参数.
            this.epsonSocket1.rec_data_refresh -= this.epsonSocket_rec_data_refresh;
            SaveConfigToIni();
        }


        private void setTag(Control cons)
        {
            foreach (Control con in cons.Controls)
            {
                con.Tag = con.Width + ":" + con.Height + ":" + con.Left + ":" + con.Top + ":" + con.Font.Size;
                if (con.Controls.Count > 0)
                    setTag(con);
            }
        }
        private void setControls(float newx, float newy, Control cons)
        {
            foreach (Control con in cons.Controls)
            {
                if (con.Tag == null) continue;
                string[] mytag = con.Tag.ToString().Split(new char[] { ':' });
                float a = Convert.ToSingle(mytag[0]) * newx;
                con.Width = (int)a;
                a = Convert.ToSingle(mytag[1]) * newy;
                con.Height = (int)(a);
                a = Convert.ToSingle(mytag[2]) * newx;
                con.Left = (int)(a);
                a = Convert.ToSingle(mytag[3]) * newy;
                con.Top = (int)(a);
                Single currentSize = Convert.ToSingle(mytag[4]) * Math.Min(newx, newy);
                con.Font = new Font(con.Font.Name, currentSize, con.Font.Style, con.Font.Unit);
                if (con.Controls.Count > 0)
                {
                    setControls(newx, newy, con);
                }
            }
        }

        void Form1_Resize(object sender, EventArgs e)
        {
            float newx = (this.Width) / X;
            float newy = this.Height / Y;
            setControls(newx, newy, this);
//             this.Text = this.Width.ToString() + " " + this.Height.ToString();

        }

        private void Point_Data_Form_Load(object sender, EventArgs e)
        {
            this.TopMost = true;
            this.Resize += new EventHandler(Form1_Resize);

            X = this.Width;
            Y = this.Height;

            String[] cnNames = { "等待位", "取件低位", "取件高位", "去毛刺低位", "去毛刺高位", "天平放件点", "天平高位", "废品位", "取样低位", "取样高位", "摆件低件", "摆件高位", "安全位", "天平取件点", "", "", "", "", "", "", "", "", "", "", "", "", "" };
            // initialize the combo box item.
            for (int i = 0; i <= 20; i++)
            {
                string str = String.Format("p({0})", i);
                if (!String.IsNullOrEmpty(cnNames[i]))
                {
                    str += "-" + cnNames[i].ToString();
                }

                combPoint.Items.Add(str);
            }
            // add the special item.
            combCmd.Items.Add("move");
            combCmd.Items.Add("go");
            combPoint.SelectedIndex = 0;
            combCmd.SelectedIndex = 0;

            setTag(this);
            Form1_Resize(new object(), new EventArgs());//x,y可在实例化时赋值,最后这句是新加的，在MDI时有用
            this.CenterToScreen();
            LoadConfigFromIni();
        }


        // 从Ini加载相应的点位配置信息.
        private void LoadConfigFromIni()
        {
            IniFile ini = new IniFile(PubConstant.config_file);
            this.txtSpeed.Text = ini.ReadString("point", "txtSpeed", "");
            this.txtStepLength.Text = ini.ReadString("point", "txtStepLength", "");
            this.combPoint.SelectedIndex = ini.ReadInteger("point", "combPoint", 0);
            this.combCmd.SelectedIndex = ini.ReadInteger("point", "combCmd", 0);

//             this.txtCurrentPointX.Text = ini.ReadString("point", "txtPointX", "");
//             this.txtCurrentPointY.Text = ini.ReadString("point", "txtPointY", "");
//             this.txtCurrentPointZ.Text = ini.ReadString("point", "txtPointZ", "");
//             this.txtCurrentPointU.Text = ini.ReadString("point", "txtPointU", "");
        }

        // 将点位配置信息保存到Ini中去.
        private void SaveConfigToIni()
        {
            IniFile ini = new IniFile(PubConstant.config_file);
            ini.WriteString("point", "txtSpeed", this.txtSpeed.Text);
            ini.WriteString("point", "txtStepLength", this.txtStepLength.Text);
            ini.WriteInteger("point", "combPoint", this.combPoint.SelectedIndex);
            ini.WriteInteger("point", "combCmd", this.combCmd.SelectedIndex);

//             ini.WriteString("point", "txtPointX", this.txtCurrentPointX.Text);
//             ini.WriteString("point", "txtPointY", this.txtCurrentPointY.Text);
//             ini.WriteString("point", "txtPointZ", this.txtCurrentPointZ.Text);
//             ini.WriteString("point", "txtPointU", this.txtCurrentPointU.Text);
        }


        // 电机开.
        private void btnMotoOn_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1 == null) return;
            if (this.epsonSocket1.IsConnection)
            {
                // 发送电机开命令.
                this.epsonSocket1.tcp_send("$setmotorson,1");
            }

            // 启动后续按钮可以工作.
            this.btnMotoOff.Enabled = true;
            // 停用所有按钮.
            this.btnXplus.Enabled = true;
            this.btnXsub.Enabled = true;
            //Y
            this.btnYplus.Enabled = true;
            this.btnYsub.Enabled = true;
            //Z
            this.btnZplus.Enabled = true;
            this.btnZsub.Enabled = true;
            //U
            this.btnUplus.Enabled = true;
            this.btnUsub.Enabled = true;
        }

        // 电机关.
        private void btnMotoOff_Click(object sender, EventArgs e)
        {
            // 关闭控制端口.
            if (this.epsonSocket1.IsConnection)
            {
                // 发送电机关命令.
                this.epsonSocket1.tcp_send("$setmotorsoff,1");
            }

            // 启动后续按钮可以工作.
            this.btnMotoOff.Enabled = false;
            // 停用所有按钮.
            this.btnXplus.Enabled = false;
            this.btnXsub.Enabled = false;
            //Y
            this.btnYplus.Enabled = false;
            this.btnYsub.Enabled = false;
            //Z
            this.btnZplus.Enabled = false;
            this.btnZsub.Enabled = false;
            //U
            this.btnUplus.Enabled = false;
            this.btnUsub.Enabled = false;
        }

        // X+
        private void btnXplus_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                int nSpeed = 0;int.TryParse(this.txtSpeed.Text, out nSpeed);
                int stepLength = 0; int.TryParse(this.txtStepLength.Text, out stepLength);
                if (stepLength <= 0) stepLength = 1;

                string cmd = String.Format("$execute,\"power low; speed {0}; accel {0},{0}; speeds {0} * 2000 / 100; accels {0} * 25000 / 100,{0} * 25000 / 100; go here+x({1})\"",
                    nSpeed, stepLength);
                this.epsonSocket1.tcp_send(cmd);
            }
        }

        // X-
        private void btnXsub_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                int nSpeed = 0; int.TryParse(this.txtSpeed.Text, out nSpeed);
                int stepLength = 0; int.TryParse(this.txtStepLength.Text, out stepLength);
                if (stepLength <= 0) stepLength = 1;

                string cmd = String.Format("$execute,\"power low; speed {0}; accel {0},{0}; speeds {0} * 2000 / 100; accels {0} * 25000 / 100,{0} * 25000 / 100; go here-x({1})\"",
                    nSpeed, stepLength);

                this.epsonSocket1.tcp_send(cmd);
            }
        }

        // Y+
        private void btnYplus_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                int nSpeed = 0; int.TryParse(this.txtSpeed.Text, out nSpeed);
                int stepLength = 0; int.TryParse(this.txtStepLength.Text, out stepLength);
                if (stepLength <= 0) stepLength = 1;

                string cmd = String.Format("$execute,\"power low; speed {0}; accel {0},{0}; speeds {0} * 2000 / 100; accels {0} * 25000 / 100,{0} * 25000 / 100; go here+y({1})\"",
                    nSpeed, stepLength);

                this.epsonSocket1.tcp_send(cmd);
            }
        }

        // Y-
        private void btnYsub_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                int nSpeed = 0; int.TryParse(this.txtSpeed.Text, out nSpeed);
                int stepLength = 0; int.TryParse(this.txtStepLength.Text, out stepLength);
                if (stepLength <= 0) stepLength = 1;

                string cmd = String.Format("$execute,\"power low; speed {0}; accel {0},{0}; speeds {0} * 2000 / 100; accels {0} * 25000 / 100,{0} * 25000 / 100; go here-y({1})\"",
                    nSpeed, stepLength);

                this.epsonSocket1.tcp_send(cmd);
            }
        }

        // Z+
        private void btnZplus_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                int nSpeed = 0; int.TryParse(this.txtSpeed.Text, out nSpeed);
                int stepLength = 0; int.TryParse(this.txtStepLength.Text, out stepLength);
                if (stepLength <= 0) stepLength = 1;

                string cmd = String.Format("$execute,\"power low; speed {0}; accel {0},{0}; speeds {0} * 2000 / 100; accels {0} * 25000 / 100,{0} * 25000 / 100; go here+z({1})\"",
                    nSpeed, stepLength);

                this.epsonSocket1.tcp_send(cmd);
            }
        }

        // Z-
        private void btnZsub_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                int nSpeed = 0; int.TryParse(this.txtSpeed.Text, out nSpeed);
                int stepLength = 0; int.TryParse(this.txtStepLength.Text, out stepLength);
                if (stepLength <= 0) stepLength = 1;

                string cmd = String.Format("$execute,\"power low; speed {0}; accel {0},{0}; speeds {0} * 2000 / 100; accels {0} * 25000 / 100,{0} * 25000 / 100; go here-z({1})\"",
                    nSpeed, stepLength);

                this.epsonSocket1.tcp_send(cmd);
            }
        }

        // U+
        private void btnUplus_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                int nSpeed = 0; int.TryParse(this.txtSpeed.Text, out nSpeed);
                int stepLength = 0; int.TryParse(this.txtStepLength.Text, out stepLength);
                if (stepLength <= 0) stepLength = 1;

                string cmd = String.Format("$execute,\"power low; speed {0}; accel {0},{0}; speeds {0} * 2000 / 100; accels {0} * 25000 / 100,{0} * 25000 / 100; go here+u({1})\"",
                    nSpeed, stepLength);

                this.epsonSocket1.tcp_send(cmd);
            }
        }

        // U-
        private void btnUsub_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                int nSpeed = 0; int.TryParse(this.txtSpeed.Text, out nSpeed);
                int stepLength = 0; int.TryParse(this.txtStepLength.Text, out stepLength);
                if (stepLength <= 0) stepLength = 1;

                string cmd = String.Format("$execute,\"power low; speed {0}; accel {0},{0}; speeds {0} * 2000 / 100; accels {0} * 25000 / 100,{0} * 25000 / 100; go here-u({1})\"",
                    nSpeed, stepLength);

                this.epsonSocket1.tcp_send(cmd);
            }
        }

        // 速度设置.
        private void btnSetSpeed_Click(object sender, EventArgs e)
        {
            string txtSpeed = this.txtSpeed.Text;
            if (String.IsNullOrEmpty(txtSpeed))
            {
                MessageBox.Show("对不起,请先输入正确的速度值!");
                return;
            }

            // 发送命令给机器人.
            if (this.epsonSocket1.IsConnection)
            {
                string cmd = String.Format("$execute,\"speed {0}\"", txtSpeed);
                this.epsonSocket1.tcp_send(cmd);
            }

            // 启动后续按钮可以工作.
            this.btnMotoOn.Enabled = true;
        }

        // 执行速度设定
        private void btnExecute_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1==null) return;
            if (this.epsonSocket1.IsConnection)
            {
                String point = this.combPoint.Text;
                if (point.IndexOf("-") > 0)
                {
                    String[] arr = point.Split('-');
                    point = arr[0];
                }

                int nSpeed = 0; int.TryParse(this.txtSpeed.Text, out nSpeed);
                if (nSpeed <= 0) nSpeed = 1;
                String cmdType = this.combCmd.Text;

                String cmd = String.Format("$execute,\"power low; speed {0}; accel {0},{0}; speeds {0} * 2000 / 100; accels {0} * 25000 / 100,{0} * 25000 / 100; {1} {2}\"", nSpeed, cmdType, point);
                this.epsonSocket1.tcp_send(cmd);
            }
        }

        // 读取点位置(&R)
        private void btnCurrentPoint_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                string point = this.combPoint.Text;
                if (point.IndexOf("-") > 0)
                {
                    String[] arr = point.Split('-');
                    point = arr[0];
                }

                String cmd = String.Format("$execute,\"print \"current; \",\"x=\", CX({0}), \";\", \"y=\", CY({0}), \";\", \"z=\", CZ({0}), \";\", \"u=\", CU({0})\"", point);
                this.epsonSocket1.tcp_send(cmd);
            }
        }

        // 读取实时点位(&T)
        private void btnRealtimePoint_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                string point = this.combPoint.Text;
                if (point.IndexOf("-") > 0)
                {
                    String[] arr = point.Split('-');
                    point = arr[0];
                }

                String cmd = String.Format("$execute,\"print \"x=\", CX(here), \";\", \"y=\", CY(here), \";\", \"z=\", CZ(here), \";\", \"u=\", CU(here)\"");
                this.epsonSocket1.tcp_send(cmd);
            }
        }

        // 示教(&T)
        private void btnTeach_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                String point = this.combPoint.Text;
                if (point.IndexOf("-") > 0)
                {
                    String[] arr = point.Split('-');
                    point = arr[0];
                }

                String cmd = String.Format("$execute,\"{0} = Here\"", point);
                this.epsonSocket1.tcp_send(cmd);
                this.isDirty = true;
            }
        }

        // 保存(&S)
        private void btnSavePoint_Click(object sender, EventArgs e)
        {
            if (this.epsonSocket1.IsConnection)
            {
                this.epsonSocket1.tcp_send("$execute,\"SavePoints \"robot1.pts\"\"");
                this.isDirty = false;
            }
        }

        // 接收到数据的响应函数.
        private void epsonSocket_rec_data_refresh(object sender, EventArgs e)
        {
            EventArgsCmd cmd = (EventArgsCmd)e;
            if (cmd == null) return;
            String str = cmd.Str;
            String[] result = str.Split(',');
            switch (cmd.Cmd)
            {
                case "execute":
                    {
                        if (str.IndexOf("execute,current,x=")>=0)
                        {
                            // 获取当前点位数据.
                            Vector3d vec = EventArgsCmd.ParseVector3d(str);
                            this.txtCurrentPointX.Text = vec.X.ToString();
                            this.txtCurrentPointY.Text = vec.Y.ToString();
                            this.txtCurrentPointZ.Text = vec.Z.ToString();
                            this.txtCurrentPointU.Text = vec.U.ToString();
                        }
                        if (str.IndexOf("execute,x=") >= 0)
                        {
                            // 获取点位数据.
                            Vector3d vec = EventArgsCmd.ParseVector3d(str);
                            this.txtRealTimePointX.Text = vec.X.ToString();
                            this.txtRealTimePointY.Text = vec.Y.ToString();
                            this.txtRealTimePointZ.Text = vec.Z.ToString();
                            this.txtRealTimePointU.Text = vec.U.ToString();
                        }
                    } break;
                default:
                    break;
            }
        }

        // 点击响应事件.
        private void RO0_Click(object sender, EventArgs e)
        {
            CheckBox obj = (CheckBox)sender;
            String ctrlName = obj.Name;
            ctrlName = ctrlName.Replace("RO", "");
            int nIndex = 0; int.TryParse(ctrlName, out nIndex);
            int nChecked = ((obj.Checked == true) ? 1 : 0);

            // 生成命令并发送.
            String cmd = String.Format("$setio,{0},{1}", nIndex, nChecked);
            if (this.epsonSocket1.IsConnection)
                this.epsonSocket1.tcp_send(cmd);

            // 更新控件显示颜色.
            obj.BackColor = Color.FromArgb(192, 192, 255);
            if (obj.Checked) obj.BackColor = Color.Red;
        }




        //         // RO5真空.
        //         private void btnRO5Empty_Click(object sender, EventArgs e)
        //         {
        //             String cmd = String.Format("$setio,5,1");
        //             if (this.epsonSocket1.IsConnection)
        //                 this.epsonSocket1.tcp_send(cmd);
        //         }
        // 

    }
}
