﻿using EPSON_SOCKET;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ZWGXZWGXPICK_SYS.DAL;

namespace ZWGXPICK_SYS
{
    public partial class Product_Data_Form : Form
    {
        // 行列最大数量.
        private static int maxItem = 301;

        private Region myRegion;
        private SolidBrush myBrush = new SolidBrush(Color.LightSeaGreen);
        private GraphicsPath[,] mlbMatrix = new GraphicsPath[301, 301];
        private defPart[,] mPart = new defPart[301, 301];   //每个产品显示情况定义.
        private defCorner[] mCorner = new defCorner[4];     //四个顶角支撑位定义.

        // 外框和中心孔.
        private GraphicsPath mlbDashBorder = new GraphicsPath(); //外框.
        private GraphicsPath mlbCenterHole = new GraphicsPath(); //中心孔.
        private GraphicsPath mlbDebugBox = new GraphicsPath();//产品调试外框.

        //      private defCalItemData[] mCalRowData = new defCalItemData[301];
        //      private defCalItemData[] mCalColData = new defCalItemData[301];

        private int boatDashType = 1;                       //舟皿类型,0:平板舟皿,1:槽形舟皿,2:圆形舟皿.
        private int productType = 1;                        //0:矩形,1:三角形.
        private static float half_sqrt3 = 1.7320508075688772f * 0.5f; //等边三角形外切圆半径公式√3/2.
        private PointF drawStartpos = new PointF(10, 10);   //仿真绘制左上角起始点位置.

        //      private double nProductItemTotalLength  = 0;        //单个产品总宽度.
        //      private double nProductItemTotalWidth  = 0;         //单个产品总高度.

        private int nPartsPerTray = 0;                      //每盘最终摆盘数量.
        private short nRowNo = 0;                      //每行数量.
        private short nColNo = 0;                      //每列数量.
        private short nColPlaceNo = 0;                      //每列摆放次数.
        private short nRowPlaceNo = 0;                      //每行摆放次数.
                                                            //      private double nLeftRightSpace = 0;                 //舟皿左右预留空隙.
                                                            //      private double nUpDownSpace = 0;                    //舟皿上下预留空隙.
                                                            //      private double nCornerWidth = 0;                    //四角支撑位宽度.
                                                            //      private double nCornerHeight = 0;                   //四角支撑位高度.
                                                            //      private double nCenterDiameter = 0;                 //中心孔直径.
                                                            //      private byte bUseXFinger = 0;                       //手指移位(目前没用)

        //      // 模拟参数
        //      private bool bOn1;
        //      private bool bOff1;
        //      private bool bOn2;
        //      private bool bOff2;
        //      private byte nSt;
        //      private byte nEnd;
        //      private byte nSt_Ctr;
        //      private byte nEnd_Ctr;

        public int id { get; set; } //当前选中行的id.
        private DataTable dataTable = null;
        public Product_Data_Form()
        {
            InitializeComponent();
            // 生成行列所有元素内容.
            for (int row = 0; row < 301; row++) {
                for (int col = 0; col < 301; col++) {
                    // 初始化所有显示单元.
                    this.mPart[row, col] = new defPart();
                    this.mlbMatrix[row, col] = new GraphicsPath();
                }
            }
        }
        private float X;

        private float Y;

        private void setTag(Control cons)
        {
            foreach (Control con in cons.Controls)
            {
                con.Tag = con.Width + ":" + con.Height + ":" + con.Left + ":" + con.Top + ":" + con.Font.Size;
                if (con.Controls.Count > 0)
                    setTag(con);
            }
        }
        private void setControls(float newx, float newy, Control cons)
        {
            foreach (Control con in cons.Controls)
            {
                if (con.Tag == null) continue;
                if (con.Name == "picMatrixLayout") continue;
                string[] mytag = con.Tag.ToString().Split(new char[] { ':' });
                float a = Convert.ToSingle(mytag[0]) * newx;
                con.Width = (int)a;
                a = Convert.ToSingle(mytag[1]) * newy;
                con.Height = (int)(a);
                a = Convert.ToSingle(mytag[2]) * newx;
                con.Left = (int)(a);
                a = Convert.ToSingle(mytag[3]) * newy;
                con.Top = (int)(a);
                Single currentSize = Convert.ToSingle(mytag[4]) * Math.Min(newx, newy);
                con.Font = new Font(con.Font.Name, currentSize, con.Font.Style, con.Font.Unit);
                if (con.Controls.Count > 0)
                {
                    setControls(newx, newy, con);
                }
            }
        }

        private void Product_Data_Form_Resize(object sender, EventArgs e)
        {
            float newx = (this.Width) / X;
            float newy = this.Height / Y;
            setControls(newx, newy, this);
            //          this.Text = this.Width.ToString() + " " + this.Height.ToString();
        }

        private void Product_Data_Form_Load(object sender, EventArgs e)
        {
            this.Resize += new EventHandler(Product_Data_Form_Resize);

            X = this.Width;
            Y = this.Height;

            setTag(this);
            Product_Data_Form_Resize(new object(), new EventArgs());//x,y可在实例化时赋值,最后这句是新加的，在MDI时有用
            this.CenterToScreen();
            BindListBox();
            UpdateDishTypeText();

            //显示第0项目参数并显示仿真界面.
            lstProdSet_SelectedIndexChanged(sender, e);
            btnEmulation_Click(sender, e);
        }

        // 读取产品数据绑定到ListBox.
        private void BindListBox()
        {
            // 显示所有日志数据.
            DataSet list = new PRODUCT_ITEM_CONFIG().GetList("");
            dataTable = list.Tables[0];


            List<String> lstProductName = new List<string>();
            foreach (DataRow row in dataTable.Rows)
            {
                String productName = row["product_name"].ToString();
                lstProductName.Add(productName);
            }

            this.lstProdSet.DataSource = dataTable;
            this.lstProdSet.DisplayMember = "product_name";
            this.lstProdSet.ValueMember = "id";
        }

        // 如果舟皿类弄发生变化,相应的参数提示文字也要变化.
        private void cb_boat_dish_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            // 更新当前的提示文本显示.
            UpdateDishTypeText();
        }

        // 更新显示文本的内容信息.
        private void UpdateDishTypeText()
        {
            if (1 == cb_boat_dish_type.SelectedIndex)
            {
                // 槽形舟皿.
                //              this.lbl_boat_dash_length.Text = "槽数(Y方向):";
                //              this.lbl_product_width.Text    = "槽间距(X方向):";
                this.txt_central_hole_radius.Enabled = false;
                this.lbl_boat_slot_num.Enabled = true;
                this.txt_boat_slot_num.Enabled = true;
                this.lbl_product_width.Text = "槽间距(Y方向):";

                // X方向取件数量只能是1.
                // this.txt_x_direction_num.Text = "1";
                // this.txt_x_direction_num.Enabled = false;
            }
            else
            {
                //              this.lbl_boat_dash_length.Text = "舟皿宽度(Y方向):";
                //              this.lbl_product_width.Text = "产品宽度(X方向):";
                this.txt_central_hole_radius.Enabled = true;
                this.lbl_boat_slot_num.Enabled = false;
                this.txt_boat_slot_num.Enabled = false;
                this.lbl_product_width.Text = "产品宽度(Y方向):";

                // X方向取件数量只能是1.
                // this.txt_x_direction_num.Enabled = true;
            }
        }

        // 产品类型发生变化.
        private void cb_product_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int product_type = this.cb_product_type.SelectedIndex;
            if (product_type == (int)eProductType.Triangle)
            {
                this.txt_x_direction_num.Text = "1";
                this.txt_y_direction_num.Text = "1";
                this.txt_x_direction_num.Enabled = false;
                this.txt_y_direction_num.Enabled = false;
            }
            else
            {
                this.txt_x_direction_num.Enabled = true;
                this.txt_y_direction_num.Enabled = true;
            }
        }

        // 选中当前产品.
        private void lstProdSet_SelectedIndexChanged(object sender, EventArgs e)
        {
            int tmpid = 0;
            int.TryParse(this.lstProdSet.SelectedValue.ToString(), out tmpid);
            this.id = tmpid;
            if ((id == 0) || (dataTable == null)) return;

            DataRow[] row = dataTable.Select("id=" + id.ToString());
            if ((row == null) || (row.Count() == 0)) return;


            DataRow item = row[0];
            SetRowToTextBox(item);

        }

        // 将Row数据直接转换成Model.
        private void SetRowToTextBox(DataRow row)
        {
            if (row == null) return;
            this.txt_product_name.Text = row["product_name"].ToString();
            int boatdishType = 0; int.TryParse(row["boat_dish_type"].ToString(), out boatdishType);
            this.cb_boat_dish_type.SelectedIndex = boatdishType;
            int productType = 0; int.TryParse(row["product_type"].ToString(), out productType);
            this.cb_product_type.SelectedIndex = productType;

            this.txt_boat_dish_length.Text = row["boat_dish_length"].ToString();
            this.txt_boat_dish_width.Text = row["boat_dish_width"].ToString();
            this.txt_boat_slot_num.Text = row["boat_slot_num"].ToString();

            this.txt_product_length.Text = row["product_length"].ToString();
            this.txt_product_width.Text = row["product_width"].ToString();
            this.txt_circular_boat_radius.Text = row["circular_boat_radius"].ToString();
            this.txt_central_hole_radius.Text = row["central_hole_radius"].ToString();
            int matrixType = 0; int.TryParse(row["matrix_type"].ToString(), out matrixType);
            int placeType = 0; int.TryParse(row["place_type"].ToString(), out placeType);
            this.cb_matrix_type.SelectedIndex = matrixType;
            this.cb_place_type.SelectedIndex = placeType;

            this.txt_x_direction_num.Text = row["x_direction_num"].ToString();
            this.txt_y_direction_num.Text = row["y_direction_num"].ToString();
            int isRotation = 0; int.TryParse(row["is_rotation"].ToString(), out isRotation);
            this.cb_is_rotation.SelectedIndex = isRotation;

            this.txt_standard_weight.Text = row["standard_weight"].ToString();
            this.txt_qualified_allow_error.Text = row["qualified_allow_error"].ToString();
            this.txt_single_weight_allow_error.Text = row["single_weight_allow_error"].ToString();
            this.txt_boat_first_point_x.Text = row["boat_first_point_x"].ToString();
            this.txt_boat_first_point_y.Text = row["boat_first_point_y"].ToString();
            this.txt_boat_dash_center_x.Text = row["boat_dash_center_x"].ToString();
            this.txt_boat_dish_center_y.Text = row["boat_dish_center_y"].ToString();
        }

        // 新建一条记录.
        private void btnNew_Click(object sender, EventArgs e)
        {
            id = 0;
            this.txt_product_name.Text = "";
            this.cb_boat_dish_type.SelectedIndex=  0;
            this.cb_product_type.SelectedIndex  =  0;
            this.txt_boat_dish_length.Text      = "1";
            this.txt_boat_dish_width.Text       = "1";
            this.txt_boat_slot_num.Text         = "1";
            this.txt_product_length.Text        = "1";
            this.txt_product_width.Text         = "1";
            this.txt_circular_boat_radius.Text  = "1";
            this.txt_central_hole_radius.Text   = "0";
            this.cb_matrix_type.SelectedIndex   =  0;
            this.cb_place_type.SelectedIndex    =  0;
            this.txt_x_direction_num.Text       = "1";
            this.txt_y_direction_num.Text       = "1";
            this.cb_is_rotation.SelectedIndex   =  0;
            this.txt_standard_weight.Text       = "0";
            this.txt_qualified_allow_error.Text = "0";
            this.txt_single_weight_allow_error.Text = "0";
            this.txt_boat_first_point_x.Text    = "0";
            this.txt_boat_first_point_y.Text    = "0";
            this.txt_boat_dash_center_x.Text    = "0";
            this.txt_boat_dish_center_y.Text    = "0";
        }

        // 将所有文件框的值获取到Model对象.
        private ZWGXPICK_SYS.Model.PRODUCT_ITEM_CONFIG TextValueToModel()
        {
            ZWGXPICK_SYS.Model.PRODUCT_ITEM_CONFIG model = new ZWGXPICK_SYS.Model.PRODUCT_ITEM_CONFIG();
            try
            {
                model.product_name = this.txt_product_name.Text;
                model.boat_dish_type = this.cb_boat_dish_type.SelectedIndex;
                model.product_type = this.cb_product_type.SelectedIndex;
                model.boat_dish_length = Utils.TryToDouble(this.txt_boat_dish_length.Text);
                model.boat_dish_width = Utils.TryToDouble(this.txt_boat_dish_width.Text);
                model.boat_slot_num = Utils.TryToInt32(this.txt_boat_slot_num.Text);

                model.product_length = Utils.TryToDouble(this.txt_product_length.Text);
                model.product_width = Utils.TryToDouble(this.txt_product_width.Text);
                model.circular_boat_radius = Utils.TryToDouble(this.txt_circular_boat_radius.Text);
                model.central_hole_radius = Utils.TryToDouble(this.txt_central_hole_radius.Text);
                model.matrix_type = this.cb_matrix_type.SelectedIndex;
                model.place_type = this.cb_place_type.SelectedIndex;
                model.x_direction_num = Utils.TryToInt32(this.txt_x_direction_num.Text);
                model.y_direction_num = Utils.TryToInt32(this.txt_y_direction_num.Text);
                model.is_rotation = this.cb_is_rotation.SelectedIndex;
                model.standard_weight = Utils.TryToDouble(this.txt_standard_weight.Text);
                model.qualified_allow_error = Utils.TryToDouble(this.txt_qualified_allow_error.Text);
                model.single_weight_allow_error = Utils.TryToDouble(this.txt_single_weight_allow_error.Text);
                model.boat_first_point_x = Utils.TryToDouble(this.txt_boat_first_point_x.Text);
                model.boat_first_point_y = Utils.TryToDouble(this.txt_boat_first_point_y.Text);
                model.boat_dash_center_x = Utils.TryToDouble(this.txt_boat_dash_center_x.Text);
                model.boat_dish_center_y = Utils.TryToDouble(this.txt_boat_dish_center_y.Text);
            }
            catch (Exception ex)
            {
                model.product_name = "";
                String info = "数据异常,可能是您在需要输入数据的地方输成了字母,请确认为重试!" + ex.StackTrace;
                MessageBox.Show(info);
            }
            //Cleanup:
            return model;
        }

        // 保存当前数据.
        private void btnSave_Click(object sender, EventArgs e)
        {


            if (MessageBox.Show("您好，确定要保存吗？", "操作确认", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                PRODUCT_ITEM_CONFIG dal = new PRODUCT_ITEM_CONFIG();
                ZWGXPICK_SYS.Model.PRODUCT_ITEM_CONFIG item = TextValueToModel();
                if (String.IsNullOrEmpty(item.product_name)) return;
                if (id == 0)
                {
                    // 新增模式.
                    id = dal.Add(item);
                    if (id == 0)
                    {
                        MessageBox.Show("保存失败,请联系管理员!");
                        return;
                    }

                    // 重新加载列表.
                    BindListBox();
                }
                else
                {
                    // 更新模式.
                    item.id = id;
                    bool bState = dal.Update(item);
                    if (!bState)
                    {
                        MessageBox.Show("更新数据失败,请联系管理员!");
                        return;
                    }

                    // 重新加载列表.
                     BindListBox();
                }
            }

        }

        // 选择当前要生产的产品.
        private void btnSelectProduct_Click(object sender, EventArgs e)
        {
            if (id != 0)
            {
                this.DialogResult = DialogResult.OK;
                this.Close();
            }
            else
            {
                MessageBox.Show("对不起,请先选择产品信息!");
            }
        }

        // 绘制阵列仿真内容.
        private void picMatrixLayout_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.SmoothingMode = SmoothingMode.HighQuality;
            // try to draw the special dash box item border now.
            e.Graphics.DrawPath(Pens.Blue, this.mlbDashBorder);
            // try to draw the special center hold item now.
            e.Graphics.DrawPath(Pens.Red, this.mlbCenterHole);

            for (int row = 0; row < maxItem; row++)
            {
                for (int col = 0; col < maxItem; col++)
                {
                    if (this.mPart[row, col].bVisible)
                    {
                        this.myRegion = new Region(this.mlbMatrix[row, col]);
                        switch (this.mPart[row, col].nColor)
                        {
                            case 0:
                                this.myBrush = new SolidBrush(Color.Blue);
                                break;

                            case 1:
                                this.myBrush = new SolidBrush(Color.Red);
                                break;

                            case 2:
                                this.myBrush = new SolidBrush(Color.Gray);
                                break;
                        }

                        e.Graphics.FillRegion(this.myBrush, this.myRegion);
                        // e.Graphics.DrawPath(Pens.Black, this.mlbMatrix[row, col]);
                    }
                }
            }
        }

        // 模拟仿真测试.
        private void btnEmulation_Click(object sender, EventArgs e)
        {
            Model.PRODUCT_ITEM_CONFIG product = TextValueToModel();
            if (product == null) return;

            // 生成渲染矩阵数据.
            this.ShowMatrix(product);

            this.txtRowNo.Text = this.nRowNo.ToString();
            this.txtColNo.Text = this.nColNo.ToString();
            this.txtPlaceNums.Text = CountProductVisible(); // (this.nColPlaceNo * this.nRowPlaceNo).ToString();
        }

        // 统计实际摆放产品个数.
        private string CountProductVisible()
        {
            short x_direction_num = (short)Utils.TryToInt32(this.txt_x_direction_num.Text);
            short y_direction_num = (short)Utils.TryToInt32(this.txt_y_direction_num.Text);
            int matrix_type = this.cb_matrix_type.SelectedIndex;
            int place_type = this.cb_place_type.SelectedIndex;

            // 对齐摆放.
            int nColPlaceNo = (short)(this.nColNo / x_direction_num);
            int nRowPlaceNo = (short)(this.nRowNo / y_direction_num);
            if ((int)ePlacementType.Chakong == place_type)
            {
                // 先X后Y.
                if (matrix_type == (int)eMatirxType.X2Y)
                {
                    
                    // 插空摆放,列支持插空摆放,一个单位是摆放两组,所以数量这里要*2.
                    if (x_direction_num>1)
                    {
                        nColPlaceNo = ((this.nColNo / (x_direction_num * 2)) * 2);
                        int lineSize = ((x_direction_num * 2) - 1);
                        if ((this.nColNo - (nColPlaceNo * x_direction_num)) >= lineSize)
                        {
                            // 最后还能多摆一行.
                            nColPlaceNo++;
                        }
                    }
                }
                //先Y后X.
                else
                {
                    // 插空摆放,行支持插空摆放,一个单位是摆放两组,所以数量这里要*2.
                    if (y_direction_num>1)
                    {
                        nRowPlaceNo = ((this.nRowNo / (y_direction_num * 2)) * 2);
                        int lineSize = ((y_direction_num * 2) - 1);
                        if ((this.nRowNo - (nRowPlaceNo * y_direction_num)) >= lineSize)
                        {
                            // 最后还能多摆一行.
                            nRowPlaceNo++;
                        }
                    }
                }
            }

            int nCount = 0;
            // 考虑上插值摆放之后的行列数量.
            int xMaxItem = nColPlaceNo * x_direction_num;
            int yMaxItem = nRowPlaceNo * y_direction_num;
            for (int row = 0; row < yMaxItem; row++)
            {
                for (int col = 0; col < xMaxItem; col++)
                {
                    if (this.mPart[row,col].bVisible )
                    {
                        nCount++;
                    }
                }
            }
        //Cleanup:
            //return (nCount*x_direction_num*y_direction_num).ToString();
            return nCount.ToString();
        }

        // 生成绘制矩阵.
        private void ShowMatrix(Model.PRODUCT_ITEM_CONFIG product)
        {
            // 重置所有产品为不可见.
            for (int row = 0; row < maxItem; row++) {
                for (int col = 0; col < maxItem; col++) {
                    // 重置所有显示图形及状态为不可见.
                    this.mPart[row, col].bVisible = false;
                    this.mlbMatrix[row, col].Reset();
                }
            }

            // 重置所有路径绘制.
            this.mlbDashBorder.Reset();
            this.mlbCenterHole.Reset();
            this.mlbDebugBox.Reset();

            // 更新当前产品类型.
            this.productType = product.product_type;
            this.boatDashType = product.boat_dish_type;

            // 获取舟皿大小.
            double dash_length = product.boat_dish_length;
            double dash_width = product.boat_dish_width;
            if (this.boatDashType == 2)
            {
                // 圆形舟皿认为长宽是相同的.
                dash_width = dash_length;
            }

            // 获取仿真窗口大小.
            int screen_width = this.picMatrixLayout.Width;
            int screen_height = this.picMatrixLayout.Height;

            // 确定绘制在仿真窗口的正中心位置.
            double left = (screen_width - dash_length) / 2.0;
            double top = (screen_height - dash_width) / 2.0;
            drawStartpos = new PointF((float)left, (float)top);

            // 根据不同的舟皿类型画出轮廓.
            if (this.boatDashType == (int)eBoatDashType.DashCircle)
            {
                // 圆形舟皿.
                this.mlbDashBorder.AddEllipse(new RectangleF((float)left, (float)top, (float)dash_length, (float)dash_length));
            }
            else
            {
                // 长方形舟皿.
                this.mlbDashBorder.AddRectangle(new RectangleF((float)left, (float)top, (float)dash_length, (float)dash_width));
            }

            // 确定是否有中心孔,有的话绘制出来.
            double hole_radius = product.central_hole_radius;
            if (hole_radius > 0)
            {
                // 画中心孔.
                double center_x = this.picMatrixLayout.Width * 0.5f;
                double center_y = this.picMatrixLayout.Height * 0.5f;
                this.mlbDashBorder.AddEllipse(new RectangleF((float)(center_x - hole_radius), (float)(center_y - hole_radius), (float)(hole_radius + hole_radius), (float)(hole_radius + hole_radius)));
            }

            // 产品大小
            float product_length = (float)product.product_length;
            float product_width = (float)product.product_width;

            // 判断当前舟皿可以放多少个产品,默认计算长方形.
            this.nPartsPerTray = 0;
            if (productType == 0)
            { 
            this.nColNo = (short)Math.Floor(dash_length / product_length);
            this.nRowNo = (short)Math.Floor(dash_width / product_width);
            }
            if (productType == 1)
            {
                // 三角形一行可以摆放多少个.
               this.nColNo = (short)Math.Floor((dash_length / (product_length * 0.5f) - 1));
              
                // 如果是三角形,那么修正产品的高度为以长度为基准.
                product_width = (half_sqrt3 * product_length);
                this.nRowNo = (short)Math.Floor(dash_width / product_width);
            }

            // 如果是槽形舟皿,列数量直接取槽数.
            if (this.boatDashType == 1) this.nRowNo = (short)product.boat_slot_num;
            // 判断一下是不是溢出了.
            float placed_width = (this.nRowNo * product_width);
            if (placed_width > product.boat_dish_width)
            {
                this.picMatrixLayout.Refresh();
                MessageBox.Show("对不起,当前摆放大小超出了舟皿大小,请调整参数重试!");
                return;
            }

            // 计算行列摆放次数.
            this.nColPlaceNo = (short)(this.nColNo / product.x_direction_num);
            this.nRowPlaceNo = (short)(this.nRowNo / product.y_direction_num);
            if (product.place_type == (int)ePlacementType.Chakong)
            {
                if (product.matrix_type == (int)eMatirxType.Y2X)
                {
                    if (product.y_direction_num > 1)
                    {
                        this.nRowPlaceNo = (short)((this.nRowNo / (product.y_direction_num * 2)) * 2);
                        int lineSize = ((product.y_direction_num * 2) - 1);
                        if ((this.nRowNo - (this.nRowPlaceNo * product.y_direction_num)) >= lineSize)
                        {
                            // 最后还能多摆一行.
                            this.nRowPlaceNo++;
                        }
                    }
                }
                else if (product.matrix_type == (int)eMatirxType.X2Y)
                {
                    if (product.x_direction_num > 1)
                    {
                        this.nColPlaceNo = (short)((this.nColNo / (product.x_direction_num * 2)) * 2);
                        int lineSize = ((product.x_direction_num * 2) - 1);
                        if ((this.nColNo - (this.nColPlaceNo * product.x_direction_num)) >= lineSize)
                        {
                            // 最后还能多摆一行.
                            this.nColPlaceNo++;
                        }
                    }
                }
            }

            /*
            // 根据摆放次数,重新修正一下行列数量.
            this.nColNo = (short)(this.nColPlaceNo * product.x_direction_num);
            this.nRowNo = (short)(this.nRowPlaceNo * product.y_direction_num);
            */

            // 计算单个产品移动步长.
            PointF step = GetStepLength(product_length, product_width, productType);
            // 根据图形类型,确定起始中心点位置,默认是矩形.
            Vector2d centerPoint = new Vector2d(drawStartpos.X + product_length * 0.5f, drawStartpos.Y + product_width * 0.5f);
            if ((int)eProductType.Triangle == productType)
            {
                // 三角形有点特殊,起始点跟矩形有所区别.
                centerPoint = new Vector2d(drawStartpos.X + product_length * 0.5f, drawStartpos.Y + (half_sqrt3 * product_length) * (2.0f / 3));
            }

            // 根据产品类型,阵列出所有点位.
            float startX = centerPoint.X;
            for (int row = 0; row < this.nRowNo; row++)
            {
                for (int col = 0; col < this.nColNo; col++)
                {
                    bool bFlip = false;
                    Vector2d tempCenter = new Vector2d(centerPoint.X, centerPoint.Y);
                    // 如果是三角形,需要根据奇偶数进行正反摆放.
                    if (((int)eProductType.Triangle == productType) && ((col % 2) == 1))
                    {
                        // 奇数列反摆,中心点往上移动1/3.
                        tempCenter.Y = tempCenter.Y - (product_width * (1.0f / 3));
                        bFlip = true;
                    }

                    // 添加绘制图形.
                    Vector2d[] points = GetSharpPoints(tempCenter, product_length, product_width, productType, bFlip);
                    AddSharpGraphics(row, col, points, productType, (float)hole_radius, this.boatDashType, (float)dash_length, (float)dash_width, bFlip);

                    // 判断是否需要绘制中心孔,这是为了测试直接绘制.
                    // this.mlbMatrix[row, col].AddEllipse(new RectangleF(new PointF(tempCenter.X - 2, tempCenter.Y - 2), new SizeF(5, 5)));

                    //添加外框绘制.
                    AddOutLineBox(mlbDebugBox, points);

                    // 移动到下一个位置.
                    centerPoint.X += step.X;

                }

                // 重置X位置并计算Y的位置.
                centerPoint.X = startX;
                centerPoint.Y += step.Y;
            }

            // modify by James 220417-去除四个边角上的摆件.
            int place_type = this.cb_place_type.SelectedIndex;
            int colSkipNum = Convert.ToInt32(product.boat_first_point_x);
            int matrix_type = this.cb_matrix_type.SelectedIndex;
            if ((colSkipNum > 0) && (colSkipNum <= product.x_direction_num) && (product.matrix_type == (int)eMatirxType.X2Y) && (product.x_direction_num > 1))
            {
                if ((int)ePlacementType.Chakong == place_type)
                    colSkipNum = 2 * product.x_direction_num;
                else
                    colSkipNum = product.x_direction_num;
            }
            int rowSkipNum = Convert.ToInt32(product.boat_first_point_y);
            if ((rowSkipNum > 0) && (rowSkipNum <= product.y_direction_num) && (product.matrix_type == (int)eMatirxType.Y2X) && (product.y_direction_num > 1))
            {
                if ((int)ePlacementType.Chakong == place_type)
                    rowSkipNum = 2 * product.y_direction_num;
                else
                    rowSkipNum = product.y_direction_num;
            }
            this.ClearCornerProduct(this.nColNo, this.nRowNo, colSkipNum, rowSkipNum);
            // modify by James end.

            this.picMatrixLayout.Refresh();
        }

        // 清除掉四个角落不摆的产品.
        private void ClearCornerProduct(int nColPlaceNo, int nRowPlaceNo, int colSkipNum,int rowSkipNum)
        {
            do
            {
                // check  if the special input parameter item value.
                if ((colSkipNum  <= 0) || (rowSkipNum  <= 0)) break;
                if ((nColPlaceNo <= 0) || (nRowPlaceNo <= 0)) break;
                if (((colSkipNum+colSkipNum)>=nColPlaceNo) || ((rowSkipNum+rowSkipNum)>=nRowPlaceNo)) break;

                // add the special four corner item value.
                List<Point> lstCorner = new List<Point>();
                lstCorner.Add(new Point(0, 0)); //LeftTop
                lstCorner.Add(new Point(nColPlaceNo - colSkipNum, 0)); //RightTop
                lstCorner.Add(new Point(nColPlaceNo - colSkipNum, nRowPlaceNo - rowSkipNum)); //RightBottom
                lstCorner.Add(new Point(0, nRowPlaceNo - rowSkipNum)); //LeftBottom

                // loop to set the special corner.
                for (int n=0;n<lstCorner.Count;n++)
                {
                    Point point = lstCorner[n];
                    for (int row = point.Y; row < point.Y + rowSkipNum; row++)
                    {
                        for (int col = point.X; col < point.X + colSkipNum; col++)
                        {
                            // 隐藏指定的元素.
                            this.mPart[row, col].bVisible = false;
                        }
                    }
                }

            } while (false);
        //Cleanup:
            return;
        }

        // 根据不同的图形,获取对应的绘制点.
        private Vector2d[] GetSharpPoints(Vector2d centerPoint, float product_length, float product_width, int productType, bool bFlip)
        {
            Vector2d[] points = null;
            do
            {
                switch (productType)
                {
                    case 1://三角形
                        {
                            // 这里为了好显示,故意缩小3像素.
                            product_length = product_length - 6;
                            Vector2d rotVector = new Vector2d(0, -(half_sqrt3 * product_length) * (2.0f / 3)); //正向摆放
                            if (bFlip)
                            {
                                rotVector = new Vector2d(0, (half_sqrt3 * product_length) * (2.0f / 3)); //反向摆放.
                            }

                            // 以中心点为中心旋转,得到三角形的三个顶.
                            points = new Vector2d[3];
                            points[0] = centerPoint + rotVector;
                            points[1] = centerPoint + rotVector.Rotate(120);
                            points[2] = centerPoint + rotVector.Rotate(240);
                        }
                        break;
                    default://默认是矩形,只取左上右下两个角.
                        {
                            // 这里为了好显示,缩小3像素,留出间距.
                            float half_length = product_length * 0.5f - 1;
                            float half_width = product_width * 0.5f - 1;
                            points = new Vector2d[4];
                            points[0] = new Vector2d(centerPoint.X - half_length, centerPoint.Y - half_width);
                            points[1] = new Vector2d(centerPoint.X + half_length, centerPoint.Y - half_width);
                            points[2] = new Vector2d(centerPoint.X + half_length, centerPoint.Y + half_width);
                            points[3] = new Vector2d(centerPoint.X - half_length, centerPoint.Y + half_width);
                        }
                        break;
                }

            } while (false);
            //Cleanup:
            return (points);
        }

        // 根据不同的图形,添加绘制图形.
        // hole_radius : 中心孔半径.
        // dashType    : 舟皿类型.
        // dashWidth   : 舟皿宽度.
        // dashHeight  : 舟皿长度.
        private bool AddSharpGraphics(int row, int col, Vector2d[] points, int productType, float hole_radius, int dashType, float dashWidth, float dashHeight,bool bFlip)
        {
            bool ret = false;
            if (points == null) return false;
            switch (productType)
            {
                case 1://三角形
                    {
                        //判断当前产品是不是落在中心孔里,如果是,那么不要摆放.
                        if ((PointsInDash(points,dashType,dashWidth,dashHeight)) && (!PointsInCenterHole(points, hole_radius)))
                        {
                            // 绘制三角形的三边边.
                            this.mlbMatrix[row, col].AddLine(new PointF(points[0].X, points[0].Y), new PointF(points[1].X, points[1].Y));
                            this.mlbMatrix[row, col].AddLine(new PointF(points[1].X, points[1].Y), new PointF(points[2].X, points[2].Y));
                            this.mlbMatrix[row, col].AddLine(new PointF(points[2].X, points[2].Y), new PointF(points[0].X, points[0].Y));
                            this.mPart[row, col].bVisible = true;
                            this.mPart[row, col].bFlip = bFlip;
                            this.nPartsPerTray++;
                            ret = true;
                        }
                    }
                    break;
                default:
                    {
                        float left = points[0].X;
                        float top = points[0].Y;
                        float width = (points[2].X - points[0].X);
                        float height = (points[2].Y - points[0].Y);
                        if ((PointsInDash(points, dashType, dashWidth, dashHeight)) && (!PointsInCenterHole(points, hole_radius)))
                        {
                            this.mlbMatrix[row, col].AddRectangle(new RectangleF(left, top, width, height));
                            this.mPart[row, col].bVisible = true;
                            this.nPartsPerTray++;
                            ret = true;
                        }
                    }
                    break;
            }
            //Cleanup:
            return ret;
        }

        // 判断所有点会不会落在圆内.
        private bool PointsInCenterHole(Vector2d[] points, float hole_radius)
        {
            bool ret = false;
            if (hole_radius <= 0) return false;
            // 中心点位置.
            float centerX = this.picMatrixLayout.Width * 0.5f;
            float centerY = this.picMatrixLayout.Height * 0.5f;

            // 循环判断线条是否相交.
            for (int i = 0; i < points.Length; i++)
            {
                float dx = Math.Abs(points[i].X - centerX);
                float dy = Math.Abs(points[i].Y - centerY);
                if ((dx * dx + dy * dy) <= (hole_radius * hole_radius))
                {
                    // 顶点与圆相交,在圆内.
                    ret = true;
                    break;
                }

                // 判断线条是否与圆相交.
                int idxNext = ((i + 1) % points.Length);
                ret = Vector2d.IsCircleIntersectLineSeg(points[i], points[idxNext], new Vector2d(centerX, centerY), hole_radius);
                if (ret) break;
            }
        //Cleanup:
            return ret;
        }
        
        // 判断产品是不是都在舟皿内,需要给出舟皿类型和舟皿大小.
        private bool PointsInDash(Vector2d[] points, int dashType, float dashWidth, float dashHeight)
        {
            
            if ((dashWidth <= 0) || (dashHeight <= 0)) return false;
            // 中心点位置.
            float centerX = this.picMatrixLayout.Width * 0.5f;
            float centerY = this.picMatrixLayout.Height * 0.5f;
            float dash_radius = dashWidth * 0.5f;
            bool bin_dash = true; //是否在舟皿内.
            // 根据舟皿类型判断产品是否在舟皿内.
            if (dashType == (int)eBoatDashType.DashCircle) //圆形舟皿.
            {
                // 循环判断线条是否相交.
                for (int i = 0; i < points.Length; i++)
                {
                    float dx = Math.Abs(points[i].X - centerX);
                    float dy = Math.Abs(points[i].Y - centerY);
                    if ((dx * dx + dy * dy) >= (dash_radius * dash_radius))
                    {
                        // 顶点与圆相交,在圆内.
                        bin_dash = false;
                        break;
                    }
                }
            }
            else //长方形.
            {
                Rectangle rect = new Rectangle((int)(centerX- (dashWidth * 0.5f)),(int)(centerY-(dashHeight * 0.5f)),(int)dashWidth,(int)dashHeight);
                // 循环判断线条是否相交.
                for (int i = 0; i < points.Length; i++)
                {
                    if (!rect.Contains(new Point((int)points[i].X, (int)points[i].Y)))
                    {
                        // 顶点与圆相交,在圆内.
                        bin_dash = false;
                        break;
                    }
                }
            }
        //Cleanup:
            return (bin_dash);
        }

        
        //添加外框绘制.
        private bool AddOutLineBox(GraphicsPath outbox, Vector2d[] points)
        {
            bool ret = false;
            if (outbox == null) return false;
            do
            {
                RectangleF rect = GetOutLineBox(points);
                if (rect != null)
                {
                    outbox.AddRectangle(rect);
                    ret = true;
                }

            } while (false);
            //Cleanup:
            return (ret);
        }

        // 获取图形的外接四边形.
        private RectangleF GetOutLineBox(Vector2d[] points)
        {
            RectangleF rect = new RectangleF(0, 0, 0, 0);
            do
            {
                float minX = 0, minY = 0, maxX = 0, maxY = 0;
                if ((points == null) || (points.Length <= 0)) break;
                minX = maxX = points[0].X;
                minY = maxY = points[0].Y;
                for (int i = 1; i < points.Length; i++)
                {
                    // 获取最小值.
                    if (points[i].X < minX) minX = points[i].X;
                    if (points[i].Y < minY) minY = points[i].Y;
                    // 获取最大值.
                    if (points[i].X > maxX) maxX = points[i].X;
                    if (points[i].Y > maxY) maxY = points[i].Y;
                }

                // 获取外接四边形的边框以及大小.
                float width = (maxX - minX);
                float height = (maxY - minY);
                rect.X = minX;
                rect.Y = minY;
                rect.Width = width;
                rect.Height = height;

            } while (false);
            //Cleanup:
            return rect;
        }

        // 获取X,Y方向步长,三角形和矩形这里会有所区别.
        private PointF GetStepLength(float product_length, float product_width, int productType)
        {
            PointF point = new PointF(product_length, product_width);
            if (productType == 1)
            {
                // 三角形.
                point.X = (product_length * 0.5f);
                point.Y = (product_length * half_sqrt3);
            }
        //Cleanup:
            return point;
        }

        // 删除当前选中配置
        private void btnDelete_Click(object sender, EventArgs e)
        {
            int productCount = this.lstProdSet.Items.Count;
            if (productCount <= 1)
            {
                MessageBox.Show("对不起,当前是最后一个产品配置参数,无法删除!", "删除");
                return;
            }

            DialogResult dr = MessageBox.Show("您好,是否确认删除当前选中的配置?", "删除", MessageBoxButtons.YesNo);
            if (dr == DialogResult.Yes)
            {
                //删除.
                PRODUCT_ITEM_CONFIG dal = new PRODUCT_ITEM_CONFIG();
                if (dal.Delete(this.id))
                {
                    // 重新加载配置项.
                    BindListBox();
                    UpdateDishTypeText();

                    //显示第0项目参数并显示仿真界面.
                    lstProdSet_SelectedIndexChanged(sender, e);
                    btnEmulation_Click(sender, e);
                }
            }
        }

        // 复制一份参数.
        private void btnCopy_Click(object sender, EventArgs e)
        {
            DialogResult dr = MessageBox.Show("您好,是否确认复制当前参数?", "复制", MessageBoxButtons.YesNo);
            if (dr == DialogResult.Yes)
            {
                PRODUCT_ITEM_CONFIG dal = new PRODUCT_ITEM_CONFIG();
                ZWGXPICK_SYS.Model.PRODUCT_ITEM_CONFIG item = TextValueToModel();
                item.product_name += "-复制1";
                if (String.IsNullOrEmpty(item.product_name)) return;

                // 新增模式.
                this.id = dal.Add(item);
                if (this.id == 0)
                {
                    MessageBox.Show("保存失败,请联系管理员!");
                    return;
                }

                // 重新加载列表.
                BindListBox();
                UpdateDishTypeText();

                //显示第0项目参数并显示仿真界面.
                lstProdSet_SelectedIndexChanged(sender, e);
                btnEmulation_Click(sender, e);
            }
        }

        // 阵列方式.
        private void cb_matrix_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            updateXYPlacedNum();
        }
        
        // 摆放方式.
        private void cb_place_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            updateXYPlacedNum();
        }

        // 更新XY轴上的摆放数量.
        private void updateXYPlacedNum()
        {
            int nMatrixType = this.cb_matrix_type.SelectedIndex;
            int nPlaceType  = this.cb_place_type.SelectedIndex;
            if (nPlaceType == 1)
            {
                if (nMatrixType == 0) //先X后Y.
                {
                    this.txt_y_direction_num.Enabled = false;
                    this.txt_x_direction_num.Enabled = true;
                    this.txt_y_direction_num.Text = "1";
                }
                else if (nMatrixType == 1) //先Y后X.
                {
                    this.txt_y_direction_num.Enabled = true;
                    this.txt_x_direction_num.Enabled = false;
                    this.txt_x_direction_num.Text = "1";
                }
            }
        }

        // 
        // 
        //         private bool Check_Circle_In_Circle(Point nPoint, short n_InRadius, short nCenter_X, short nCenter_Y, short nRadius)
        //         {
        //             return ((Math.Pow((double)(nPoint.X - nCenter_X), 2.0) + Math.Pow((double)(nPoint.Y - nCenter_Y), 2.0)) < Math.Pow((double)((short)(nRadius - n_InRadius)), 2.0));
        //         }
        // 
        //         // 判断是不是在圆以内.
        //         private bool Check_Circle_NotIn_Circle(Point nPoint, short n_InRadius, short nCenter_X, short nCenter_Y, short nRadius)
        //         {
        //             return ((nRadius == 0) || ((Math.Pow((double)(nPoint.X - nCenter_X), 2.0) + Math.Pow((double)(nPoint.Y - nCenter_Y), 2.0)) > Math.Pow((double)((short)(nRadius + n_InRadius)), 2.0)));
        //         }
        // 
        //         private bool Check_Point_In_Circle(Point nPoint, short nCenter_X, short nCenter_Y, short nRadius)
        //         {
        //             return ((Math.Pow((double)(nPoint.X - nCenter_X), 2.0) + Math.Pow((double)(nPoint.Y - nCenter_Y), 2.0)) < Math.Pow((double)nRadius, 2.0));
        //         }
        // 
        //         // 判断三角形或矩形不在圆之内.
        //         private bool Check_Point_NotIn_Circle(Point[] nPoint, short nCenter_X, short nCenter_Y, short nRadius, short nPointNo)
        //         {
        //             bool flag = false;
        //             try
        //             {
        //                 if (nRadius == 0)
        //                 {
        //                     return true;
        //                 }
        //                 if (nPointNo == 4)
        //                 {
        //                     return (((((nPoint[0].X > ((short)(nCenter_X + nRadius))) & (nPoint[1].X > ((short)(nCenter_X + nRadius)))) | ((nPoint[0].X < ((short)(nCenter_X - nRadius))) & (nPoint[1].X < ((short)(nCenter_X - nRadius))))) | ((nPoint[0].Y > ((short)(nCenter_Y + nRadius))) & (nPoint[2].Y > ((short)(nCenter_Y + nRadius))))) | ((nPoint[0].Y < ((short)(nCenter_Y - nRadius))) & (nPoint[2].Y < ((short)(nCenter_Y - nRadius)))));
        //                 }
        //                 if (nPointNo == 3)
        //                 {
        //                     if (!(((((nPoint[0].X > ((short)(nCenter_X + nRadius))) & (nPoint[1].X > ((short)(nCenter_X + nRadius)))) | ((nPoint[0].X < ((short)(nCenter_X - nRadius))) & (nPoint[1].X < ((short)(nCenter_X - nRadius))))) | ((nPoint[0].Y > ((short)(nCenter_Y + nRadius))) & (nPoint[2].Y > ((short)(nCenter_Y + nRadius))))) | ((nPoint[0].Y < ((short)(nCenter_Y - nRadius))) & (nPoint[2].Y < ((short)(nCenter_Y - nRadius))))))
        //                     {
        //                         short num11 = (short)(nPointNo - 1);
        //                         for (short i = 0; i <= num11; i = (short)(i + 1))
        //                         {
        //                             double num2;
        //                             if (i < (nPointNo - 1))
        //                             {
        //                                 // 求三角形和圆是否相交,没看懂...
        //                                 double x    = Math.Pow(Math.Pow(nPoint[i].X - nCenter_X, 2.0) + Math.Pow(nPoint[i].Y - nCenter_Y, 2.0), 0.5);
        //                                 double num5 = Math.Pow(Math.Pow(nPoint[i + 1].X - nCenter_X, 2.0) + Math.Pow(nPoint[i + 1].Y - nCenter_Y, 2.0), 0.5);
        //                                 double num4 = Math.Pow(Math.Pow(nPoint[i].X - nPoint[i + 1].X, 2.0) + Math.Pow(nPoint[i].Y - nPoint[i + 1].Y, 2.0), 0.5);
        //                                 double num6 = Math.Pow(((Math.Pow(num4, 2.0) + Math.Pow(num5, 2.0)) - Math.Pow(x, 2.0)) / ((2.0 * num4) * num5), 2.0);
        //                                 num2 = Math.Sqrt(1.0 - num6) * num5;
        //                             }
        //                             else
        //                             {
        //                                 double x     = Math.Pow(Math.Pow(nPoint[0].X - nCenter_X, 2.0) + Math.Pow(nPoint[0].Y - nCenter_Y, 2.0), 0.5);
        //                                 double num9  = Math.Pow(Math.Pow(nPoint[i].X - nCenter_X, 2.0) + Math.Pow(nPoint[i].Y - nCenter_Y, 2.0), 0.5);
        //                                 double num8  = Math.Pow(Math.Pow(nPoint[0].X - nPoint[i].X, 2.0) + Math.Pow(nPoint[0].Y - nPoint[i].Y, 2.0), 0.5);
        //                                 double num10 = Math.Pow(((Math.Pow(num8, 2.0) + Math.Pow(num9, 2.0)) - Math.Pow(x, 2.0)) / ((2.0 * num8) * num9), 2.0);
        //                                 num2 = Math.Sqrt(1.0 - num10) * num9;
        //                             }
        // 
        //                             // check if the number content is the value.
        //                             // if ((double.IsNaN(num2)) || (num2 < nRadius))
        //                             if (num2 < nRadius)
        //                             {
        //                                 return false;
        //                             }
        //                         }
        //                     }
        // 
        //                     return true;
        //                 }
        //                 flag = false;
        //             }
        //             catch (Exception ex)
        //             {
        //                 MessageBox.Show("判断三角形或矩形不在圆内出错:" + ex.StackTrace);
        //             }
        // 
        //             return flag;
        //         }


        //         // 判断开关状态.
        //         private void Chk_OnOff(byte nSn, byte bIsOn)
        //         {
        //             switch (bIsOn)
        //             {
        //                 case 0:
        //                     if (!this.bOn1)
        //                     {
        //                         this.bOn1 = true;
        //                         this.nSt = (byte)(nSn + 1);
        //                     }
        //                     if (this.bOff1 && !this.bOn2)
        //                     {
        //                         this.bOn2 = true;
        //                         this.nEnd_Ctr = nSn;
        //                     }
        //                     break;
        // 
        //                 case 1:
        //                     if (this.bOn1 && !this.bOff1)
        //                     {
        //                         this.bOff1 = true;
        //                         this.nEnd = nSn;
        //                     }
        //                     if (this.bOn2 && !this.bOff2)
        //                     {
        //                         this.bOff2 = true;
        //                         this.nSt_Ctr = (byte)(this.nEnd + 1);
        //                         this.nEnd = nSn;
        //                     }
        //                     break;
        // 
        //                 case 2:
        //                     if (this.bOn2 && !this.bOff2)
        //                     {
        //                         this.bOff2 = true;
        //                         this.nSt_Ctr = (byte)(this.nEnd + 1);
        //                         this.nEnd = nSn;
        //                     }
        //                     if (!this.bOn1)
        //                     {
        //                         this.nSt = 0;
        //                         this.nEnd = 0;
        //                     }
        //                     break;
        //             }
        //         }

        //         private Point Get_Circle_OffSetPoint(Point nPoint, short nRadius)
        //         {
        //             return new Point
        //             {
        //                 Y = (int)Math.Round((double)(nPoint.Y - (1.7320508075688772 * nRadius))),
        //                 X = nPoint.X - nRadius
        //             };
        //         }
    }
}
